from rlab_jax.envs.wrapper import (NEnvs, RescaleAction)


def env_from_cfg(cfg, n_envs):
    return NEnvs(cfg, n_envs)


def create_single_env_from_cfg(cfg):
    if cfg.type == "dm-control":
        from rlab_jax.envs.dm_control_env import DMControlSuiteEnv

        env = DMControlSuiteEnv(name=cfg.name.split("-")[0], task=cfg.name.split("-")[1], image_observations=cfg.image_observations, dataset_name=cfg.name)

    elif cfg.type == "d4rl":
        import d4rl
        import d4rl.gym_mujoco
        import d4rl.hand_manipulation_suite
        import gym as old_gym
        import numpy as np
        from rlab_jax import keygen
        import jax

        from .wrapper import GymToGymnasiumEnv, GymToGymnasiumEnvNoResetSeed, TimeLimitFix

        if cfg.name in ["pen-cloned-v0", "hammer-cloned-v0", "relocate-cloned-v0", "door-cloned-v0"]:
            env = GymToGymnasiumEnvNoResetSeed(TimeLimitFix(old_gym.make(cfg.name).env))
        else:
            if hasattr(cfg, "env_kwargs"):
                env = GymToGymnasiumEnv(TimeLimitFix(old_gym.make(cfg.name, **cfg.env_kwargs).env))
            else:
                env = GymToGymnasiumEnv(TimeLimitFix(old_gym.make(cfg.name).env))

        seed = int(jax.random.randint(key=keygen(), minval=0, maxval=10 ** 5, shape=(1,)))
        env.reset(seed=seed)
        env.action_space.seed(seed)

    else:
        raise ValueError("Unknown env type!")

    # rescale action to [-1, 1] and update action_space
    env = RescaleAction(env)

    if hasattr(cfg, "action_repeat"):
        from .wrapper import ActionRepeat

        env = ActionRepeat(env, cfg.action_repeat)

    return env
