import jax.numpy as jnp
import numpy as np

from rlab_jax.evaluation.evaluation import Evaluation


class RewardEvaluation(Evaluation):
    _EVALUATION_TYPE = "RewardEvaluation"
    _BATCH_ONLY = True

    def __init__(self, writer, env, *args, **kwargs):
        self.writer = writer
        self.env = env

    def __call__(self, batch, step, prefix, *args, **kwargs):
        # TODO: include batch.mask? Env might be ending early

        reward = batch.reward.sum(0).mean()
        self.writer.add_scalar(prefix + "reward/real_world", reward.item(), step)

        if self.env.spec.entry_point is not None and self.env.spec.entry_point.split(".")[0] == "d4rl":
            from d4rl.infos import REF_MAX_SCORE, REF_MIN_SCORE
            norm_reward = (reward - REF_MIN_SCORE[self.env.spec.id]) / (
                    REF_MAX_SCORE[self.env.spec.id] - REF_MIN_SCORE[self.env.spec.id]) * 100

            self.writer.add_scalar(prefix + "norm_reward/real_world", norm_reward.item(), step)

        if hasattr(self.env, "normalize_reward"):
            norm_reward = self.env.normalize_reward(reward)
            self.writer.add_scalar(prefix + "norm_reward/real_world", norm_reward.item(), step)
