import os
from abc import abstractmethod
import equinox as eqx
import omegaconf
from rlab_jax.rlab_dataclasses.other import ModulesState


class Experiment:
    subclasses = {}

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        if hasattr(cls, "_EXPERIMENT_TYPE"):
            cls.subclasses[cls._EXPERIMENT_TYPE] = cls

    @classmethod
    def create(cls, cfg: omegaconf.DictConfig, *args, **kwargs):
        if cfg.experiment.type not in cls.subclasses:
            raise ValueError(f"Bad experiment type {cfg.experiment.type}")

        agent = cls.subclasses[cfg.experiment.type].from_cfg(cfg, *args, **kwargs)

        return agent

    @abstractmethod
    def from_cfg(cls, cfg: omegaconf.DictConfig, **_ignored):
        pass

    @abstractmethod
    def train(self):
        pass

    def save_checkpoint(self,  modules_state, step="", name=None):
        if name is None:
            name = f"checkpoint{step}"

        # checkpoint_path = os.path.join(self.log_dir, f"{name}.eqx")
        # eqx.tree_serialise_leaves(checkpoint_path, modules_state)
        # TODO implement checkpoint saving here

    def load_checkpoint(self,  modules_state, path):
        print(f"Loading checkpoint: {path}")
        # TODO implement checkpoint loading here
        #return eqx.tree_deserialise_leaves(path, modules_state)
        return modules_state
