from abc import abstractmethod

import omegaconf


class Agent:
    subclasses = {}

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        if hasattr(cls, "_AGENT_TYPE"):
            cls.subclasses[cls._AGENT_TYPE] = cls

    @classmethod
    def create(cls, cfg: omegaconf.DictConfig, *args, **kwargs):
        if cfg.type not in cls.subclasses:
            raise ValueError(f"Bad agent type {cfg.type}")

        agent = cls.subclasses[cfg.type].from_cfg(cfg, *args, **kwargs)

        return agent

    @classmethod
    @abstractmethod
    def from_cfg(cls, cfg: omegaconf.DictConfig, n_inputs, n_actions, **_ignored):
        pass

    @abstractmethod
    def act(self, state):
        """Used to create training rollouts. deterministic=False"""
        pass

    @abstractmethod
    def evaluate(self, state):
        """Used to create validation rollouts. deterministic=True"""
        pass

    @abstractmethod
    def training_step(self, batch):
        pass
