from abc import abstractmethod


class Reward:
    """Provides a reward function."""

    subclasses = {}

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        if hasattr(cls, "_REWARD_TYPE"):
            cls.subclasses[cls._REWARD_TYPE] = cls

    @classmethod
    def create(cls, cfg, *args, **kwargs):
        if cfg.type not in cls.subclasses:
            raise ValueError(f"Bad reward type {cfg.type}")

        return cls.subclasses[cfg.type].from_cfg(cfg, *args, **kwargs)

    @classmethod
    @abstractmethod
    def from_cfg(cls, **_ignored):
        pass
