from abc import abstractmethod
from typing import Optional, Tuple

import omegaconf


class SequenceModel:
    subclasses = {}

    def __init__(self):
        super().__init__()
        self._n_initial_obs = None

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        if hasattr(cls, "_SEQUENCE_MODEL_TYPE"):
            cls.subclasses[cls._SEQUENCE_MODEL_TYPE] = cls

    @classmethod
    def create(cls, cfg: omegaconf.DictConfig, *args, **kwargs):
        if cfg.type not in cls.subclasses:
            raise ValueError(f"Bad sequence model type: {cfg.type}")

        return cls.subclasses[cfg.type].from_cfg(cfg, *args, **kwargs)

    @classmethod
    @abstractmethod
    def from_cfg(cls, cfg: omegaconf.DictConfig, n_observations, n_actions, **_ignored):
        """Instantiate object from config."""
