import chex
import equinox as eqx
import jax.numpy as jnp
import optax


@chex.dataclass
class TrainState:
    module: eqx.Module
    optim: optax.GradientTransformation
    optim_state: optax.OptState

    def apply_updates(self, grads):
        updates, self.optim_state = self.optim.update(grads, self.optim_state, eqx.filter(self.module, eqx.is_array))
        self.module = eqx.apply_updates(self.module, updates)
        return self


@chex.dataclass
class AnnealingTrainState(TrainState):
    annealing: jnp.array


@chex.dataclass
class ScalerTrainState(TrainState):
    scaler: eqx.Module

@chex.dataclass
class ScalerAnnealingTrainState(AnnealingTrainState):
    scaler: eqx.Module

