import chex
import equinox as eqx
import jax.numpy as jnp
import optax

from rlab_jax.distributions import (multivariate_normal_diag,
                                    tanh_multivariate_normal_diag)


@chex.dataclass
class DreamRolloutTransition:
    state: jnp.array
    next_state: jnp.array
    action: jnp.array
    reward: jnp.array
    done: jnp.array
    was_done: jnp.array = None
    latent_action: jnp.array = None


@chex.dataclass
class Transition:
    action: jnp.array
    observation: jnp.array
    reward: jnp.array
    done: jnp.array = None
    next_observation: jnp.array = None
    latent_action: jnp.array = None
    latent_state: jnp.array = None
    next_latent_state: jnp.array = None
    mask: jnp.array = None

    @property
    def observation_trajectory(self):
        return jnp.concatenate((jnp.expand_dims(self.observation[0], 0), self.next_observation), axis=0)

    @property
    def action_trajectory(self):
        return jnp.concatenate((jnp.zeros([1] + list(self.action.shape[1:])), self.action), axis=0)