import argparse
import hydra
from omegaconf import OmegaConf, DictConfig

if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("config", help="name of a config file inside hydra folder")
    args = parser.parse_args()

    from rlab_jax.utils.cfg import load_cfg
    cfg = load_cfg(config_name=args.config)

    if hasattr(cfg.meta, "wandb_key") and cfg.meta.wandb_key is not None:
        import wandb

        wandb.login(key=cfg.meta.wandb_key)
        wandb.init(project=cfg.meta.project,
                   name=cfg.meta.group + "-" + cfg.meta.name,
                   config=OmegaConf.to_container(cfg, resolve=True))
        wandb.tensorboard.patch(tensorboard_x=True)

    if hasattr(cfg.experiment, "seed") and cfg.experiment.seed is not None:
        import os
        import random
        import numpy as np
        from rlab_jax.utils.keygen import keygen

        keygen.set(cfg.experiment.seed)
        os.environ["PYTHONHASHSEED"] = str(cfg.experiment.seed)
        random.seed(cfg.experiment.seed)
        np.random.seed(cfg.experiment.seed)

    from rlab_jax.experiments.experiment import Experiment
    experiment = Experiment.create(cfg)
    experiment.train()
