import os

import yaml
from hydra.core.global_hydra import GlobalHydra
from omegaconf import OmegaConf

from hydra import compose, initialize_config_dir
from rlab_jax import RLAB_DIR


def load_cfg(config_name):
    GlobalHydra.instance().clear()
    initialize_config_dir(config_dir=os.path.join(RLAB_DIR, "hydra"))
    return compose(config_name=config_name)


def load_cfg_from_experiment_dir(experiment_dir):
    initialize_config_dir(config_dir=experiment_dir)
    return compose(config_name="cfg.yaml")


def save_cfg(log_dir, cfg):
    path = os.path.join(log_dir, "cfg.yaml")

    with open(path, "w") as stream:
        try:
            yaml.safe_dump(OmegaConf.to_container(cfg), stream)
        except yaml.YAMLError as exc:
            print(exc)
