import jax
import tqdm

from rlab_jax.utils.keygen import keygen
from rlab_jax_projects.latent_actions.d4rl_datasets import get_offline_dataset
from rlab_jax_projects.latent_actions.experiments.latent_action_mf_off_policy import \
    LatentActionModelFreeOffPolicyExperiment

class OfflineLatentActionModelFreeOffPolicyExperiment(LatentActionModelFreeOffPolicyExperiment):
    _EXPERIMENT_TYPE = "OfflineLatentActionModelFreeOffPolicyExperiment"

    def __init__(self, model_training_epochs, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.model_training_epochs = model_training_epochs

    def train(self):
        modules_state = self.initial_state()

        self.train_rollout.memory, memory_state = get_offline_dataset(env=self.train_env, memory=self.train_rollout.memory, writer=self.writer, prng_key=keygen())

        key, subkey = jax.random.split(keygen())

        for epoch in tqdm.tqdm(range(1, self.epochs + 1), desc="epochs"):
            step = epoch * self.train_rollouts_per_epoch * self.env_steps_per_rollout

            if epoch < self.model_training_epochs + 1:
                # MODEL TRAINING
                num_updates_per_epoch = int(
                    self.train_rollouts_per_epoch
                    * self.env_steps_per_rollout
                    * self.train_steps_ratio
                )
                key, subkey = jax.random.split(key)
                modules_state, train_logs = self.model_training_epoch(
                    memory_state=memory_state,
                    modules_state=modules_state,
                    num_updates_per_epoch=num_updates_per_epoch,
                    key=subkey,
                )

                for k, v in train_logs.items():
                    self.writer.add_scalar(k, v.item(), step)

            else:
                # AGENT TRAINING
                num_updates_per_epoch = int(
                    self.train_rollouts_per_epoch
                    * self.env_steps_per_rollout
                    * self.train_steps_ratio
                )
                key, subkey = jax.random.split(key)
                modules_state, train_logs = self.agent_training_epoch(
                    memory_state=memory_state,
                    modules_state=modules_state,
                    num_updates_per_epoch=num_updates_per_epoch,
                    key=subkey,
                )

                for k, v in train_logs.items():
                    self.writer.add_scalar(k, v.item(), step)

            # VAL
            if epoch % self.val_every_n_epochs == 0:

                # VAL ROLLOUT
                key, subkey = jax.random.split(key)
                batch = self.val_rollout.generate(
                    agent_state=modules_state.agent_state,
                    model_state=modules_state.model_state,
                    steps_per_rollout=self.env_steps_per_rollout,
                    key=subkey,
                )

                self.best_checkpoint(batch=batch,
                                     epoch=epoch,
                                     modules_state=modules_state)

                if self.evaluation is not None:
                    key, subkey = jax.random.split(key)
                    self.evaluation(
                        agent_state=modules_state.agent_state,
                        model_state=modules_state.model_state,
                        batch=batch,
                        step=step,
                        epoch=epoch,
                        key=subkey,
                    )

            if epoch % self.checkpoint_every_n_epochs == 0:
                self.save_checkpoint(step=step, modules_state=modules_state)
