import jax.numpy as jnp

from rlab_jax.distributions import (multivariate_normal_diag,
                                    tanh_multivariate_normal_diag)
from rlab_jax.modules import Module


class Gauss(Module):
    _MODULE_TYPE = "Gauss"
    _dist_type = "normal"
    n_outputs: int

    def __init__(self, n_outputs, **ignored):
        self.n_outputs = n_outputs
        super().__init__()

    def __call__(self, x):
        mean = jnp.zeros(list(x.shape)[:-1] + [self.n_outputs])
        std = jnp.ones(list(x.shape)[:-1] + [self.n_outputs])
        return multivariate_normal_diag(mean=mean, std=std)


class TanhGauss(Gauss):
    _MODULE_TYPE = "TanhGauss"
    _dist_type = "tanh_normal"

    def __call__(self, x):
        mean = jnp.zeros(list(x.shape)[:-1] + [self.n_outputs])
        std = jnp.ones(list(x.shape)[:-1] + [self.n_outputs]) * 0.5
        return tanh_multivariate_normal_diag(mean=mean, std=std)
