import equinox as eqx
import jax
import jax.numpy as jnp

from rlab_jax.modules import Module
from rlab_jax.modules.rewards import StochasticLearnedReward
from rlab_jax.rlab_dataclasses import StateActionRandomVariable, TrainState


class LatentActionStochasticLearnedReward(StochasticLearnedReward):
    _MODULE_TYPE = "LatentActionStochasticLearnedReward"

    def __call__(
        self,
        reward_model: Module,
        action: jnp.array,
        next_state: StateActionRandomVariable,
        key: jax.random.PRNGKey,
        return_dist=False,
    ):
        if isinstance(next_state, StateActionRandomVariable):
            inputs = jnp.concatenate((next_state.state.sample, next_state.action.sample), -1)
        else:
            inputs = jnp.concatenate((next_state.sample, action), -1)

        reward_dist = reward_model(inputs)

        if return_dist:
            return reward_dist

        return reward_dist.sample(key)
