import jax.numpy as jnp
from rlab_jax.modules import Module
from rlab_jax.rlab_dataclasses import Transition


class MeanStdScaler(Module):
    mean: jnp.array
    std: jnp.array

    def __init__(self, mean=None, std=None, shape=None):
        super().__init__()
        if mean is not None:
            self.mean = mean
            self.std = std
        else:
            self.mean = jnp.zeros(shape)
            self.std = jnp.ones(shape)

    def fit(self, data: Transition, model_state):
        mean = jnp.mean(data.observation, axis=(0, 1))
        std = jnp.std(data.observation, axis=(0, 1))

        model_state.scaler = MeanStdScaler(mean=mean, std=std)

        return model_state

    @staticmethod
    def transform(x, scaler):
        return (x - scaler.mean) / scaler.std

    @staticmethod
    def inverse_transform(x, scaler):
        return x * scaler.std + scaler.mean


class NoScaler(Module):

    def fit(self, data: Transition, model_state):
        return model_state

    @staticmethod
    def transform(x, scaler):
        return x

    @staticmethod
    def inverse_transform(x, scaler):
        return x
